function x = browsweep(L, b, bs);
%--------------------------------------------------------------------------------
% Solve a lower triangular system Lx = b using a block row sweep algorithm with
% blocksize bs. If the blocksize bs is not provided, a default of bs = 4 is used.
%
% The right hand side b and solution vector x can be aliased.
%
% See the colsweep.m file for comments about the convention of using capital letters
% for indexing vectors (vs. indexing scalars)
%--------------------------------------------------------------------------------

checklower = false; % Test for L being lower triangular or not
x = b;

% Look for dumb mistakes
if isempty(L)
    disp('Error in browsweep; L should be nonempty')
    x = [];
    return
end

[m,n] = size(L);
if (m~=n | length(b) ~= m)
    disp('Error in browsweep; L must be square and conformal with b')
    return
end

if checklower % Make sure that L is lower triangular
    upper_part = triu(L,1);
    up_norm    = norm(upper_part,1);
    if (up_norm > 1.0e-14)
        disp('Error in trisol1; L must be lower triangular')
        disp(['Upper part has one-norm of ', num2str(up_norm)]);
        disp(['The cut-off limit is n*eps = ', num2str(n*eps)]);
        return
    end
end

if (nargin < 3)
    bs = 4;
end

nblocks = ceil(n/bs);
for ki = 1:nblocks
    I = (ki-1)*bs+1:min(n,ki*bs);
    for kj = 1:ki-1
        J = (kj-1)*bs+1:min(n,kj*bs);
        x(I) = x(I) - L(I,J)*x(J);    % L(I,J) may not be square here
    end
    x(I) = rowsweep(L(I,I), x(I));  % <--- equivalent to x(I) = L(I,I)\b(I)
end

