function L = setl(n, diagdom, unitL, nightmare)
%-------------------------------------------------------------------------------------
% Set up a lower triangular matrix. The nighmare case causes awful blow-up, which can
% be worsened by making the first subdiagonal have 10s instead of 2s on it. Otherwise,
% the subdiagonal entries are uniformly random in [0.5, 0.5]. 
%
% If unitL = true, the diagonal entries are equal to 1.0
%
% If diagdom = true, make L strongly diagonally dominant.
%
% Other options can be set by commenting/uncommenting.
%-------------------------------------------------------------------------------------

    if (nargin < 2), diagdom   = false; end
    if (nargin < 3), unitL     = true;  end
    if (nargin < 4), nightmare = false; end

    if nightmare
        % For stress testing, make L be unit with 2's on first subdiagonal
        L = zeros(n,n);
        L(1:n+1:end) = ones(1,n);
        L(2:n+1:n^2) = 2.0;
        return
    end

    % L = tril(randn(n,n));
    L = tril(rand(n,n)-0.5*ones(n,n));

    if diagdom
        L(1:n+1:end) = L(1:n+1:end) + n;
        return  % diagonal dominance is stronger than unit condition
    end

    if unitL
        L(1:n+1:end) = ones(1,n);
    end
return
